#encoding: UTF-8 
=begin
/***************************************************************************
 *   ©2019-2019 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end

require 'optparse'
require 'optparse/time'
require 'ostruct'
require 'date'
require_relative 'translating'
require_relative 'logging'
require_relative 'version'

class ArgParser 
  extend Translating
  extend Logging

  @log = self::init_logger

=begin
# unused for the time
        def self.trl_month(localized_month)
          month = %w~
          January
          February
          March
          April
          May
          June
          July
          August
          September
          October
          November
          December
          ~.detect {|mn| trl(mn) == localized_month }
         return month
        end
=end

=begin
  Translate a string like '23' to an integer like 23
=end
  def self::atoi(string)
    ciphers = (48..57)
    a_num = string.bytes.collect {|b| b if ciphers === b}.compact
    if( ! a_num.empty? && a_num.length == string.length)
      number = 0
      # To facilitate work with the index, I avoid reverse_each and rather
      # reverse the array right away.
      a_num.reverse!
      a_num.each_with_index do |b, i|
        # Do the multiplication with 10s as taught in your first programming lessons.
        number += (b-48) * (10**i)
      end
      return number
    else
      return nil
    end 
  end

  # Returns a structure describing the options.
  # param_procs are closures to handle specific options, as needed by specific
  # executables. There is a strategy-pattern lurking somewhere, but this
  # appears to be the wrong place to apply it. The really cool detail here is
  # the hash, not the procs.
  def self.parse(args, param_procs = {})
    # The options specified on the command line will be collected in <b>options</b>.
    # I set default values here. 
    options = OpenStruct.new
    options.sheet = 1
    options.backup = false
    options.delref = nil

    op = OptionParser.new do |opts|
      opts.banner = trl("Usage") << ":\t" << "%s <options>" %($0) <<"\n"

      opts.separator ""
      opts.separator trl("Specific options") << ":"

      # An attempt to treat numerical sheet-ids and sheet-names alike on the
      # command-line. This can be wrong but I like it for the time.
      opts.on('-' << trl('s'), '--' << trl('sheet') << '=' << trl('SHEET'), trl('The sheet to modify. Name or numerical ID.')) do |sheet|
        # set to integer- or string-id
        options.sheet = atoi(sheet)
        options.sheet ||= sheet
      end

      opts.on('-' << trl('b'), '--' << trl('backup'), TrueClass, trl('Create a backup-copy of the original spreadsheet file.') ) do 
        options.backup = true
      end

      opts.on('-' << trl('d'), '--' << trl('delete EXPR'), trl('Delete all lines which conform to the expression.') ) do |expr|
        # handle the delete argument as needed by the calling executable.
        if(param_procs[:delete] && param_procs[:delete].lambda?)
           options.delete = param_procs[:delete].call(expr)
        else
          ex = OptionParser::ParseError.new()
          ex.reason = trl("Option %s is known but no way to handle it.") %'-d/--delete'
          raise ex
        end
      end

      opts.separator ""
      opts.separator (trl("Common options")) << ':'

      # show help
      opts.on_tail('-' << trl("h"), '--' << trl("help"), (trl("Show this message") ) ) do
        puts opts
        exit true
      end

      # show version information.
      opts.on_tail(("-v"), ("--version"), (trl("Show version and program information") )) do
        puts "\t#{$0} #{VERSION}"
        puts "\t#{COPYRIGHT}"
        exit true
      end
    end
    begin
      op.parse!(args)
    rescue OptionParser::ParseError => ex
      @log.error(trl('Option(s)') << ' ' << ex.args.join(', ' ) << ': ' << trl(ex.reason) )
      @log.info(trl("Call with option -h or --help to see the option summary.") )
      exit false
    end
    options
  end  # parse()
end  

