#encoding: UTF-8 
=begin
/***************************************************************************
 *   ©2019-2019 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         * *
Free Software Foundation, Inc.,                                       * *   59
Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end

require_relative 'workbook'
require_relative 'logging'
require_relative 'translating'
require 'singleton'
require 'tmpdir'
require 'zip'

# 1 Object of this class manipulates 1 spreadsheet-file at a time  
class Spreadsheet
  include Singleton
  include Translating
  extend Logging

  @@log = init_logger(STDOUT)

  def initialize
    @log = @@log
    @log.debug("initialize")
    # REMINDER call the attr_writer on options
    @options = nil
  end
  
  # Set the file and do the necessary.
  def file=(fl)
    dirname = File::absolute_path(File::dirname(fl) )
    @file = File::absolute_path(fl)
    @log.debug('spreadsheet file is ' << @file << ", dirname is " << dirname)
    
    # create a copy of the original file, if demanded.
    # FOR THE TIME BEING, THE COPY IS ALWAYS CREATED.
    
    # if(@options.backup)
    File::binwrite(File.join(dirname.dup, "BAK_" << File.basename(fl)), File::binread(@file ) )
    #end

    # read the zipped spreadsheet
    find_entries
  end
  
  def sheet(attr = 1)
    if(attr.respond_to?(:to_str))
      @sheets.detect {|s| s.name == attr } 
    elsif (attr.respond_to?(:to_i) )
      @sheets.detect {|s| s.id == attr}
    end
  end

  def update_sheet(sheet)
    @log.debug('updating sheet ' << sheet.zip_entry << ' in file ' << @file)
    # Zip::File.replace needs a file but looks cool, otherwise.
    # Create the file.
    td = Dir.mktmpdir
    tfile = File.join(td, 't_sheet')
    File.write(tfile, sheet.table)

    # substitute the original with the temporary file.
    Zip::File.open(@file) do |ofl|
      ofl.replace(sheet.zip_entry, tfile)
    end
    # remove temporary file and -directory
    File.unlink(tfile)
    Dir.rmdir(td)
  end

  # If needed. 
  def cleanup
    # Close the InputStream
    @spr_in.close
  end

  attr_reader :sheets
  attr_writer :options

  private

  # unused.
  def shared_string(value)
    @shared_strings.css('t').index(strings.css('t').detect { |s| s.text == value })
  end

  def find_entries
    @spr_in = Zip::InputStream.open(@file)
    # workbook
    @wb = nil
    # SharedStrings
    @shared_strings = nil 
    # Sheets
    wb_sheets = nil
    @sheets = Array.new
    @log.debug('looking for entries')
    while(entry = @spr_in.get_next_entry)
      case(entry.name)
      when 'xl/workbook.xml'
        @log.debug("\tfound " << entry.name)
        @wb = Workbook.new(entry.get_input_stream.read)
        wb_sheets = @wb.document.css('sheet')
      when 'xl/sharedStrings.xml'
        @log.debug("\tfound " << entry.name)
        @shared_strings = Nokogiri::XML::parse(entry.get_input_stream.read )
      end
    end
    # @spr_in.rewind does only jump to the beginning of the 'current zip entry', 
    # NOT the beginning of the InputStream. F1. Help me if you can...
    @spr_in.close
    @spr_in = Zip::InputStream.open(@file)
    # ... on second thought ... No. This should be okay.

    i = 0
    while(entry = @spr_in.get_next_entry)
      # This index-thing is fishy. But so is the structure of the zip-file as
      # compared to the list of sheets in workbook.xml. If the convention is
      # broken, then neither Ruby nor my program bear the responsibility.
      # ... And what are my alternatives to match xml text with file-system
      # objects?
      if entry.name == 'xl/worksheets/sheet' << (i+1).to_s << '.xml'
        @log.debug("\tfound " << entry.name)
        ws = wb_sheets[i]
        i+= 1 
        sheet = Sheet.new(ws, entry.name, entry.get_input_stream.read, @shared_strings)
        @sheets.push(sheet)
      end
    end
    # Yeah. I'm an animal.
  end
end # class

#### TEST #####
if __FILE__ == $0

  require 'ostruct'

  fl = FileManip.instance()
  fl.options = OpenStruct.new(:backup => true)
  fl.file='../test/3cols.pmdx'

end

#n EOF
