#encoding: UTF-8
=begin
/***************************************************************************
 *   ©2011-2019, Michael Uplawski <michael.uplawski@uplawski.eu>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         * 
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end


RD = File.expand_path(File.dirname(__FILE__) ) +  File::SEPARATOR if !defined?(RD)

require 'yaml'
require_relative 'file_checking'


# A way to produce translated text in a ruby-program.
# Translations are read from a file "translations" in the program folder.
module Translating
  # 2-letter language-code
  @@lang = nil 
  # If present, a file containing a 2-letter language code. This can be the
  # default language and overrides the environment variable LANG.
  @@lang_file = format("%s%s", RD, 'LANG')  
  # The currently defined translations
  @@tr = YAML::load_file("#{RD}translations")

  # find the current language-setting and return it.
  def self.language()
    if( !File.exist?(@@lang_file) || !File::readable?(@@lang_file) || File::size(@@lang_file) < 2)
      # Only if there is no LANG-file
      if @@lang == nil
        # ... try the environment setting
        r = ENV['LANG']
        if(r)
          @@lang = r[0, 2]
        end
      end
    else
      # ... else read the code from the file
      File::open(@@lang_file, 'r') {|f| @@lang = f.readline}
      @@lang.chomp!.downcase! if @@lang
    end
    # ... and if nothing works, set language to English.
    @@lang ||= 'en' if !@@lang
  end

  # Translate a string to the currently set langage.
  # Note that the argument may contain a format-string
  # in sprintf-syntax.
  def self.trl(t)
    Translating::language()
    lt = @@tr[t]
    if(lt)
      lt = lt[@@lang]
    else
      # File.open('/tmp/mtf', 'a+') {|f| f << t << "\n"}
      puts "\nTRANSLATION MISSING: \"" << t << "\""
    end
    lt ||= t
    return lt
  end

  # Translate a string to the currently set langage. 
  # Note that the argument may contain a format-string
  # in sprintf-syntax.
  def trl(t )
    Translating::trl(t)
  end

  # 2019
  # Translate in the given string only those words for which a translation
  # is found.
  # This is useful for mixed content like dates: "Friday, 8. July 2011".
  # As the order changes with the locale, alphabetic content is matched and the
  # remainder discarded.
  def self::trl_words(str)
    n_str = str.clone
    loop do 
      m = str.match(/[[:alpha:]]+/)
      break if !m
      ### allow the loop to break
      str.gsub!(m.to_s, '')
      n_str.gsub!(m.to_s, trl(m.to_s))
    end
    return n_str
  end
  # Translate in the given string only those words for which a translation
  # exists.
  # This is useful for mixed content like dates: "Friday, 8. July 2011".
  # As the order changes with the locale, alphabetic content is matched and the
  # remainder discarded.
  def trl_words(str)
    Translating::trl_words(str)
  end

end

##### TEST
if __FILE__ == $0
  Translating::trl_words('12. september 2019, or so, not the 31! As that would be an Error!')
end
